import os, sys, logging, warnings
from typing import Optional, LiteralString
from functools import wraps
from hrenpack.listwork import key_in_dict


if sys.version_info >= (3, 13):
    from warnings import deprecated
else:
    class deprecated:
        def __init__(self, message: LiteralString, /, *,
                     category: type[Warning] = DeprecationWarning,
                     stacklevel: int = 1):
            self.message = message
            self.category = category
            self.stacklevel = stacklevel

        def _decorate_function(self, func):
            @wraps(func)
            def wrapper(*args, **kwargs):
                warnings.warn(self.message, self.category, self.stacklevel)
                return func(*args, **kwargs)

            wrapper.__deprecated__ = True
            return wrapper

        def _decorate_class(self, cls: type):
            init = cls.__init__

            @wraps(init)
            def new_init(self, *args, **kwargs):
                warnings.warn(self.message, self.category, self.stacklevel)
                return init(self, *args, **kwargs)

            cls.__init__ = new_init
            cls.__deprecated__ = True
            cls.__deprecated_message__ = self.message
            return cls

        def __call__(self, decorated):
            if sys.version_info >= (3, 13):
                return warnings.deprecated(self.message, category=self.category, stacklevel=self.stacklevel)(decorated)
            elif isinstance(decorated, type):
                return self._decorate_class(decorated)
            else:
                return self._decorate_function(decorated)


def confirm(inp_text: str = "Вы уверены, что хотите выполнить эту программу?"):
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            result = input(inp_text + "\nВведите y, д или 1, если да, или n, н или 0, если нет\n")
            while True:
                if result in ('y', 'Y', "д", "Д", "1"):
                    return func(*args, **kwargs)
                elif result in ('n', 'N', "н", "Н", "0"):
                    break
                else:
                    result = input(inp_text + "\nВведите y, д или 1, если да, или n, н или 0, если нет\n")

        return wrapper
    return decorator


def non_print(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        with open(os.devnull, 'w') as devnull:
            old_stdout = sys.stdout
            sys.stdout = devnull
            try:
                f = func(*args, **kwargs)
            finally:
                sys.stdout = old_stdout
                return f
    return wrapper


def args_kwargs(**kwargs):
    args_name = kwargs.get('args_name', 'args')
    kwargs_name = kwargs.get('kwargs_name', 'kwargs')
    copy_args = kwargs.get('copy_args', True)
    copy_kwargs = kwargs.get('copy_kwargs', True)
    del_kwargs = kwargs.get('del_kwargs', True)

    def decorator(func):
        @wraps(func)
        def wrapper(*args, **key_args):
            if key_in_dict(key_args, args_name) and copy_args:
                args = [*args, *key_args[args_name]]
                if del_kwargs:
                    del key_args[args_name]
            if key_in_dict(key_args, kwargs_name) and copy_kwargs:
                key_args = {**key_args, **key_args[kwargs_name]}
                if del_kwargs:
                    del key_args[kwargs_name]
            return func(*args, **key_args)
        return wrapper
    return decorator


def debug_logging(start_message: str = '', end_message: str = ''):
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            if start_message:
                logging.debug(start_message)
            output = func(*args, **kwargs)
            if end_message:
                logging.debug(end_message)
            return output
        return wrapper
    return decorator


def method(func):
    return func


def multi_decorator(*decorators):
    """Декораторы применяются слева направо"""
    def decorator(func):
        for dec in decorators:
            func = dec(func)
        return func
    return decorator



import uuid
from datetime import datetime
from typing import Literal
from hrenpack.strwork import remove_extra_spaces
from hrenpack.filework import TextFile


class HostsFile(TextFile):
    comment_letter = '#'

    def __init__(self):
        super().__init__(r'C:/Windows/System32/drivers/etc/hosts')

    def read_hosts(self) -> dict:
        output = dict()
        for line in self.read_lines():
            if not line or line.startswith(self.comment_letter):
                continue
            ip, domain = remove_extra_spaces(line).split()
            output[ip] = domain
        return output

    def add_host(self, ip, domain, dont_backup: bool = False):
        entry = f'{ip} {domain}\n'
        self.add_data(entry)
        if not dont_backup:
            self.backup()

    def remove_host(self, domain, dont_backup: bool = False):
        hosts = self.read_hosts()
        with open(self.path, 'w', encoding='utf-8') as file:
            for ip, host in hosts.items():
                if domain == host:
                    file.write(f'{ip} {host}\n')
        if not dont_backup:
            self.backup()

    def backup(self):
        self.copy(f'hosts_backups/hosts-{datetime.now()}')



from pip_setuptools import setup, clean, find_packages, requirements

desc = '\n'.join(("Универсальная библиотека python для большинства задач", 'A universal python library for most tasks'))

BASE_REQUIREMENTS = requirements()
IMAGE_REQUIREMENTS = requirements('image_requirements.txt')
FLASK_REQUIREMENTS = requirements('flask_requirements.txt')
FILETYPE_REQUIREMENTS = requirements('filetype_requirements.txt')
BASE_DEV_REQUIREMENTS = BASE_REQUIREMENTS + requirements('dev_requirements.txt')
FULL_DEV_REQUIREMENTS = BASE_DEV_REQUIREMENTS + IMAGE_REQUIREMENTS + FLASK_REQUIREMENTS + FILETYPE_REQUIREMENTS

REQUIREMENTS = {
    'base': BASE_REQUIREMENTS,
    'image': BASE_REQUIREMENTS + IMAGE_REQUIREMENTS,
    'flask': BASE_REQUIREMENTS + FLASK_REQUIREMENTS,
    'filetype': BASE_REQUIREMENTS + FILETYPE_REQUIREMENTS,
    'dev': BASE_DEV_REQUIREMENTS,
    'dev_base': BASE_DEV_REQUIREMENTS,
    'dev_image': BASE_DEV_REQUIREMENTS + IMAGE_REQUIREMENTS,
    'dev_flask': BASE_DEV_REQUIREMENTS + FLASK_REQUIREMENTS,
    'dev_filetype': BASE_DEV_REQUIREMENTS + FILETYPE_REQUIREMENTS,
    'all': BASE_REQUIREMENTS + FLASK_REQUIREMENTS + IMAGE_REQUIREMENTS + FILETYPE_REQUIREMENTS,
    'dev_all': FULL_DEV_REQUIREMENTS,
    'dev_full': FULL_DEV_REQUIREMENTS,
}

clean()
setup(
    name='hrenpack',
    version='2.5.1',
    author_email='magilyas.doma.09@list.ru',
    author='Маг Ильяс DOMA (MagIlyasDOMA)',
    description=desc,
    license='MIT',
    url='https://github.com/MagIlyasDOMA/hrenpack',
    packages=find_packages(),
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Operating System :: Microsoft :: Windows :: Windows 10',
        'Operating System :: Microsoft :: Windows :: Windows 11',
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: 3.14",
        "Framework :: Django",
        "Framework :: Django :: 5.2",
        "Natural Language :: English",
        "Natural Language :: Russian",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Utilities",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Security :: Cryptography",
        "Topic :: Text Processing :: Markup",
        "Topic :: Text Processing :: Markup :: HTML",
    ],
    platforms=[
        "Windows",
        "Windows 10",
        "Windows 11",
        "Windows Server 2019+",
    ],
    project_urls=dict(
        Source="https://github.com/MagIlyasDOMA/hrenpack",
        Documentation="https://magilyasdoma.github.io/hrenpack/documentation.html",
    ),
    python_requires='>=3.10',
    install_requires=BASE_REQUIREMENTS,
    extras_require=REQUIREMENTS,
    include_package_data=True
)
